﻿using Infragistics.UltraChart.Core;
using Infragistics.UltraChart.Core.Primitives;
using Infragistics.UltraChart.Core.Util;
using Infragistics.UltraChart.Resources.Appearance;
using Infragistics.UltraChart.Shared.Styles;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace ChartToStart
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            ultraChart1.FillSceneGraph += ultraChart1_FillSceneGraph;
        }

        void ultraChart1_FillSceneGraph(object sender, Infragistics.UltraChart.Shared.Events.FillSceneGraphEventArgs e)
        {

            if (this.ultraChart1.LineChart.ChartText.Count > 0)
            {
                // not finished
                #region approach with ChartText collection being used
                List<Text> chartTextPrimitives = this.FindChartTextPrimitives(e.SceneGraph);
                
                Box background = null;
                Rectangle bounds = Rectangle.Empty;
                SizeF labelSize = SizeF.Empty;
                Rectangle shiftedBounds = Rectangle.Empty;
                Box backgroundShadow = null;

                foreach (Text text in chartTextPrimitives)
                {
                    labelSize = Platform.GetLabelSizePixels(text.GetTextString(), text.labelStyle);
                    bounds = new Rectangle(text.bounds.Location, labelSize.ToSize());
                    
                    background = new Box(bounds);

                    background.PE.Fill = Color.PaleGoldenrod;
                    background.PE.StrokeWidth = 1;
                    background.PE.Stroke = Color.Black;
                    background.PE.ElementType = PaintElementType.SolidFill;

                    shiftedBounds = bounds;
                    shiftedBounds.Offset(2, 2);
                    backgroundShadow = new Box(shiftedBounds);
                    backgroundShadow.PE.Fill = Color.Gray;
                    backgroundShadow.PE.StrokeWidth = 0;
                    backgroundShadow.PE.ElementType = PaintElementType.SolidFill;

                    e.SceneGraph.Add(backgroundShadow);
                    e.SceneGraph.Add(background);
                }

                #endregion // approach with ChartText collection being used
            }
            else
            {
                #region approach without ChartText collection being used

                List<Polyline> lines = new List<Polyline>();

                foreach (Primitive p in e.SceneGraph)
                {
                    if (p is Polyline)
                    {
                        lines.Add(p as Polyline);
                    }
                }

                if (lines.Count < 1) return;

                foreach (Polyline line in lines)
                {
                    foreach (DataPoint point in line.points)
                    {
                        Text label = new Text();
                        double dblValue = double.NaN;
                        if (double.TryParse(point.Value.ToString(), out dblValue))
                        {

                            dblValue = dblValue / 100;
                            label.SetTextString(dblValue.ToString("P"));
                        }

                        SizeF labelSize = Platform.GetLabelSizePixels(label.GetTextString(), label.labelStyle);

                        Rectangle bounds = new Rectangle(
                            point.point.X - (int)labelSize.Width / 2,
                            point.point.Y - (int)labelSize.Height - 5,
                            (int)labelSize.Width,
                            (int)labelSize.Height);

                        label.bounds = bounds;

                        Box background = new Box(bounds);

                        background.PE.Fill = Color.PaleGoldenrod;
                        background.PE.StrokeWidth = 1;
                        background.PE.Stroke = Color.Black;
                        background.PE.ElementType = PaintElementType.SolidFill;

                        Rectangle shiftedBounds = bounds;
                        shiftedBounds.Offset(2, 2);
                        Box backgroundShadow = new Box(shiftedBounds);
                        backgroundShadow.PE.Fill = Color.Gray;
                        backgroundShadow.PE.StrokeWidth = 0;
                        backgroundShadow.PE.ElementType = PaintElementType.SolidFill;

                        e.SceneGraph.Add(backgroundShadow);
                        e.SceneGraph.Add(background);
                        e.SceneGraph.Add(label);

                    }
                }

                #endregion // approach without ChartText collection being used
            }
        }

        #region Helper Method
        private List<Text> FindChartTextPrimitives(SceneGraph sceneGraph)
        {
            List<Text> textPrimitives = new List<Text>(5);
            // start from end and walk backwards to find polyline.
            int index = -1;
            for (int i = sceneGraph.Count - 1; i > 0; i--)
            {
                if (sceneGraph[i] is Polyline)
                {
                    index = i;
                    break;
                }
            }
            // walk backwards from polyline to find text primitives
            if (index != -1)
            {
                for (int i = index; i > 0; i--)
                {
                    // stop when a graphics context is encountered.
                    if (sceneGraph[i] is GraphicsContext)
                    {
                        break;
                    }

                    if (sceneGraph[i] is Text)
                    {
                        textPrimitives.Add(sceneGraph[i] as Text);
                    }
                }
            }

            return textPrimitives;
        }
        #endregion // Helper Method

        private void Form1_Load(object sender, EventArgs e)
        {
           DataTable dt = new DataTable();
            dt.Columns.Add("col1", typeof(double));
            dt.Columns.Add("col2", typeof(string));
            dt.Rows.Add(new object[] { 10, "1/1/2008" });
            dt.Rows.Add(new object[] { 40, "1/2/2008" });
            dt.Rows.Add(new object[] { 22, "1/3/2008" });
            dt.Rows.Add(new object[] { 66, "1/4/2008" });
            ultraChart1.Data.SwapRowsAndColumns = true;
            ultraChart1.Data.DataSource = dt;
            this.ultraChart1.ChartType = ChartType.LineChart;
            this.ultraChart1.DataBind();

            this.ultraChart1.Axis.Y.Labels.ItemFormatString = "<DATA_VALUE:00>";
            this.ultraChart1.Axis.Y.TickmarkStyle = Infragistics.UltraChart.Shared.Styles.AxisTickStyle.DataInterval;
            this.ultraChart1.Axis.Y.RangeType = AxisRangeType.Automatic;
            this.ultraChart1.Axis.Y.TickmarkInterval = 5;
            this.ultraChart1.Axis.Y.Margin.Near.MarginType = LocationType.Pixels;
            this.ultraChart1.Axis.Y.Margin.Near.Value = 10;
            this.ultraChart1.Axis.Y.Margin.Far.MarginType = LocationType.Pixels;
            this.ultraChart1.Axis.Y.Margin.Far.Value = 10;

            this.ultraChart1.Axis.X.Margin.Near.MarginType = LocationType.Pixels;
            this.ultraChart1.Axis.X.Margin.Near.Value = 10;
            this.ultraChart1.Axis.X.Margin.Far.MarginType = LocationType.Pixels;
            this.ultraChart1.Axis.X.Margin.Far.Value = 10;

            //Simpler way to add text 

            //ChartTextAppearance cta = new ChartTextAppearance();
            //cta.FontColor = Color.Black;
            //cta.Visible = true;
            //cta.Row = -2;
            //cta.Column = -2;
            //cta.ItemFormatString = "<DATA_VALUE:00.0000>%";
            //cta.HorizontalAlign = StringAlignment.Center;
            //cta.VerticalAlign = StringAlignment.Far;
            //ultraChart1.LineChart.ChartText.Add(cta);
        
            //ShadowEffect shadowEfect = new ShadowEffect();

            //shadowEfect.Angle = 45;

            //shadowEfect.Color = Color.Aqua;

            //shadowEfect.Depth = 20;

            //ultraChart1.Effects.Add(shadowEfect);
           
        }
    }
}
